﻿

local moduleName = "HealthBar"



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Health Bar",

	["Agro Color"] = true,
	["Color the healthbar of the person with agro red."] = true,

	["Texture"] = true,
	["The texture used for the health bar."] = true
} end)

L:RegisterTranslations("koKR", function() return {
	[moduleName] = "생명력 바",

	["Agro Color"] = "어그로 색상",
	["Color the healthbar of the person with agro red."] = "어그로를 획득한 캐릭터의 생명력바 색상을 붉은색으로 표시합니다.",

	["Texture"] = "텍스처",
	["The texture used for the health bar."] = "생명력바에 텍스처를 사용합니다."
} end)

----------------------------------
--      Local Declaration      --
----------------------------------

local SharedMedia     = AceLibrary("SharedMedia-1.0")
local SharedMediaType = SharedMedia.MediaType and SharedMedia.MediaType.STATUSBAR or "statusbar"

local texture

local agro = { }



----------------------------------
--      Module Declaration      --
----------------------------------

local plugin = BunchOfBars:NewModule(moduleName, "AceEvent-2.0")

plugin.revision = tonumber(("$Revision: 39544 $"):match("%d+"))

plugin.options = {
	name = L[moduleName],
	args = {
		agrocolor = {
			type     = "toggle",
			name     = L["Agro Color"],
			desc     = L["Color the healthbar of the person with agro red."],
			disabled = function() return not plugin.core:HasModule("Agro") end,
            get      = "GetSetColorAgro",
            set      = "GetSetColorAgro"
		},
		texture = {
			type = 'text',
			name = L["Texture"],
			desc = L["The texture used for the health bar."],
			get  = "GetSetTexture",
			set  = "GetSetTexture",
			validate = SharedMedia:List(SharedMediaType),
		}
	}
}

plugin.defaultDB = {
	position = 2,

	agrocolor = true,
	texture   = "Charcoal"
}



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnEnable()
	if self.core:HasModule("Agro") and self.db.profile.agrocolor then
		agro = self.core:GetModule("Agro").agro
	end

	texture = SharedMedia:Fetch(SharedMediaType, self.db.profile.texture)

	self:RegisterBucketEvent({"UNIT_HEALTH", "UNIT_MAXHEALTH", "BunchOfBars_Agro"}, 0.05, "UpdateUnits")
end


function plugin:UpdateBar(bar, unit)
	local m = UnitHealthMax(unit)
	local c = UnitHealth(unit)

	bar:SetMinMaxValues(0, m)
	bar:SetValue(c)

	if agro[unit] then
		if agro[unit] < 1.5 then
			bar:SetStatusBarColor(1, 0.5, 0)
		else
			bar:SetStatusBarColor(1, 0, 0)
		end
	else
		local p = c / m
		local r, g

		if p >= 0.5 then
			r, g = (1 - p) * 2, 1
		else
			r, g = 1, p * 2
		end

		bar:SetStatusBarColor(r, g, 0)
	end
end


function plugin:OnCreate(frame)
	local bar = CreateFrame("StatusBar", nil, frame)
	bar:SetStatusBarTexture(texture)
	bar:SetFrameLevel(3)
	bar:SetWidth(100)
	bar:SetHeight(18)
	bar:Show()

	local back = frame:CreateTexture(nil, "OVERLAY")
	back:SetTexture("Interface/Tooltips/UI-Tooltip-Background")
	back:SetVertexColor(0, 0, 0, 1)
	back:ClearAllPoints()
	back:SetAllPoints(bar)
	back:Show()

	return bar
end


function plugin:OnUpdate(frame, bar)
	self:UpdateBar(bar, frame.unit)
end


function plugin:UpdateUnits(units)
	for unit in pairs(units) do
        if self.core.frames[unit] then
			self:UpdateBar(self.core.frames[unit].parts[self.name], unit)
		end
    end
end


function plugin:GetSetColorAgro(v)
	if type(v) == "nil" then return self.db.profile.agrocolor end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.agrocolor ~= v then
		self.db.profile.agrocolor = v

		if self.db.profile.agrocolor then
			agro = self.core:GetModule("Agro").agro
		else
			agro = { }
		end
	end
end


function plugin:GetSetTexture(v)
	if type(v) == "nil" then return self.db.profile.texture end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.texture ~= v then
		self.db.profile.texture = v

		texture = SharedMedia:Fetch(SharedMediaType, self.db.profile.texture)
		
		for _,frame in pairs(self.core.frames) do
			frame.parts[self.name]:SetStatusBarTexture(texture)
		end
	end
end
